@@ -13,7 +13,7 @@ def unicode2utf8(arg):
     """
 
     if PY2 and isinstance(arg, unicode):
-        return arg.encode('utf-8')
+        return arg.encode("utf-8")
     return arg
 
 
@@ -24,5 +24,5 @@ def utf8tounicode(arg):
     """
 
     if PY2 and isinstance(arg, str):
-        return arg.decode('utf-8')
+        return arg.decode("utf-8")
     return arg
