@@ -7,7 +7,7 @@ import psycopg2.extensions as ext
 import sqlparse
 import pgspecial as special
 import select
-from psycopg2.extensions import POLL_OK, POLL_READ, POLL_WRITE
+from psycopg2.extensions import POLL_OK, POLL_READ, POLL_WRITE, make_dsn
 from .packages.parseutils.meta import FunctionMetadata, ForeignKey
 from .encodingutils import unicode2utf8, PY2, utf8tounicode
 
@@ -24,7 +24,7 @@ ext.register_type(ext.new_type((2249,), "RECORD", ext.UNICODE))
 
 # Cast bytea fields to text. By default, this will render as hex strings with
 # Postgres 9+ and as escaped binary in earlier versions.
-ext.register_type(ext.new_type((17,), 'BYTEA_TEXT', psycopg2.STRING))
+ext.register_type(ext.new_type((17,), "BYTEA_TEXT", psycopg2.STRING))
 
 # TODO: Get default timeout from pgclirc?
 _WAIT_SELECT_TIMEOUT = 1
@@ -50,6 +50,10 @@ def _wait_select(conn):
             conn.cancel()
             # the loop will be broken by a server error
             continue
+        except select.error as e:
+            errno = e.args[0]
+            if errno != 4:
+                raise
 
 
 # When running a query, make pressing CTRL+C raise a KeyboardInterrupt
@@ -63,17 +67,19 @@ def register_date_typecasters(connection):
     Casts date and timestamp values to string, resolves issues with out of
     range dates (e.g. BC) which psycopg2 can't handle
     """
+
     def cast_date(value, cursor):
         return value
+
     cursor = connection.cursor()
-    cursor.execute('SELECT NULL::date')
+    cursor.execute("SELECT NULL::date")
     date_oid = cursor.description[0][1]
-    cursor.execute('SELECT NULL::timestamp')
+    cursor.execute("SELECT NULL::timestamp")
     timestamp_oid = cursor.description[0][1]
-    cursor.execute('SELECT NULL::timestamp with time zone')
+    cursor.execute("SELECT NULL::timestamp with time zone")
     timestamptz_oid = cursor.description[0][1]
     oids = (date_oid, timestamp_oid, timestamptz_oid)
-    new_type = psycopg2.extensions.new_type(oids, 'DATE', cast_date)
+    new_type = psycopg2.extensions.new_type(oids, "DATE", cast_date)
     psycopg2.extensions.register_type(new_type)
 
 
@@ -94,7 +100,7 @@ def register_json_typecasters(conn, loads_fn):
     """
     available = set()
 
-    for name in ['json', 'jsonb']:
+    for name in ["json", "jsonb"]:
         try:
             psycopg2.extras.register_json(conn, loads=loads_fn, name=name)
             available.add(name)
@@ -113,7 +119,9 @@ def register_hstore_typecaster(conn):
     """
     with conn.cursor() as cur:
         try:
-            cur.execute("SELECT 'hstore'::regtype::oid")
+            cur.execute(
+                "select t.oid FROM pg_type t WHERE t.typname = 'hstore' and t.typisdefined"
+            )
             oid = cur.fetchone()[0]
             ext.register_type(ext.new_type((oid,), "HSTORE", ext.UNICODE))
         except Exception:
@@ -124,29 +132,29 @@ class PGExecute(object):
 
     # The boolean argument to the current_schemas function indicates whether
     # implicit schemas, e.g. pg_catalog
-    search_path_query = '''
-        SELECT * FROM unnest(current_schemas(true))'''
+    search_path_query = """
+        SELECT * FROM unnest(current_schemas(true))"""
 
-    schemata_query = '''
+    schemata_query = """
         SELECT  nspname
         FROM    pg_catalog.pg_namespace
-        ORDER BY 1 '''
+        ORDER BY 1 """
 
-    tables_query = '''
+    tables_query = """
         SELECT  n.nspname schema_name,
                 c.relname table_name
         FROM    pg_catalog.pg_class c
                 LEFT JOIN pg_catalog.pg_namespace n
                     ON n.oid = c.relnamespace
         WHERE   c.relkind = ANY(%s)
-        ORDER BY 1,2;'''
+        ORDER BY 1,2;"""
 
-    databases_query = '''
+    databases_query = """
         SELECT d.datname
         FROM pg_catalog.pg_database d
-        ORDER BY 1'''
+        ORDER BY 1"""
 
-    full_databases_query = '''
+    full_databases_query = """
         SELECT d.datname as "Name",
             pg_catalog.pg_get_userbyid(d.datdba) as "Owner",
             pg_catalog.pg_encoding_to_char(d.encoding) as "Encoding",
@@ -154,87 +162,158 @@ class PGExecute(object):
             d.datctype as "Ctype",
             pg_catalog.array_to_string(d.datacl, E'\n') AS "Access privileges"
         FROM pg_catalog.pg_database d
-        ORDER BY 1'''
+        ORDER BY 1"""
 
-    socket_directory_query = '''
+    socket_directory_query = """
         SELECT setting
         FROM pg_settings
         WHERE name = 'unix_socket_directories'
-    '''
+    """
 
-    def __init__(self, database, user, password, host, port, dsn, **kwargs):
-        self.dbname = database
-        self.user = user
-        self.password = password
-        self.host = host
-        self.port = port
-        self.dsn = dsn
-        self.extra_args = {k: unicode2utf8(v) for k, v in kwargs.items()}
-        self.connect()
+    view_definition_query = """
+        WITH v AS (SELECT %s::pg_catalog.regclass::pg_catalog.oid AS v_oid)
+        SELECT nspname, relname, relkind,
+               pg_catalog.pg_get_viewdef(c.oid, true),
+               array_remove(array_remove(c.reloptions,'check_option=local'),
+                            'check_option=cascaded') AS reloptions,
+               CASE
+                 WHEN 'check_option=local' = ANY (c.reloptions) THEN 'LOCAL'::text
+                 WHEN 'check_option=cascaded' = ANY (c.reloptions) THEN 'CASCADED'::text
+                 ELSE NULL
+               END AS checkoption
+        FROM pg_catalog.pg_class c
+        LEFT JOIN pg_catalog.pg_namespace n ON (c.relnamespace = n.oid)
+        JOIN v ON (c.oid = v.v_oid)"""
 
-    def connect(self, database=None, user=None, password=None, host=None,
-                port=None, dsn=None, **kwargs):
+    function_definition_query = """
+        WITH f AS
+            (SELECT %s::pg_catalog.regproc::pg_catalog.oid AS f_oid)
+        SELECT pg_catalog.pg_get_functiondef(f.f_oid)
+        FROM f"""
 
-        db = (database or self.dbname)
-        user = (user or self.user)
-        password = (password or self.password)
-        host = (host or self.host)
-        port = (port or self.port)
-        dsn = (dsn or self.dsn)
-        kwargs = (kwargs or self.extra_args)
-        pid = -1
-        if dsn:
-            if password:
-                dsn = "{0} password={1}".format(dsn, password)
-            conn = psycopg2.connect(dsn=unicode2utf8(dsn))
-            cursor = conn.cursor()
-        else:
-            conn = psycopg2.connect(
-                database=unicode2utf8(db),
-                user=unicode2utf8(user),
-                password=unicode2utf8(password),
-                host=unicode2utf8(host),
-                port=unicode2utf8(port),
-                **kwargs)
+    version_query = "SELECT version();"
 
-            cursor = conn.cursor()
+    def __init__(
+        self,
+        database=None,
+        user=None,
+        password=None,
+        host=None,
+        port=None,
+        dsn=None,
+        **kwargs
+    ):
+        self._conn_params = {}
+        self.conn = None
+        self.dbname = None
+        self.user = None
+        self.password = None
+        self.host = None
+        self.port = None
+        self.server_version = None
+        self.extra_args = None
+        self.connect(database, user, password, host, port, dsn, **kwargs)
+        self.reset_expanded = None
 
-        conn.set_client_encoding('utf8')
-        if hasattr(self, 'conn'):
+    def copy(self):
+        """Returns a clone of the current executor."""
+        return self.__class__(**self._conn_params)
+
+    def connect(
+        self,
+        database=None,
+        user=None,
+        password=None,
+        host=None,
+        port=None,
+        dsn=None,
+        **kwargs
+    ):
+
+        conn_params = self._conn_params.copy()
+
+        new_params = {
+            "database": database,
+            "user": user,
+            "password": password,
+            "host": host,
+            "port": port,
+            "dsn": dsn,
+        }
+        new_params.update(kwargs)
+
+        if new_params["dsn"]:
+            new_params = {"dsn": new_params["dsn"], "password": new_params["password"]}
+
+            if new_params["password"]:
+                new_params["dsn"] = make_dsn(
+                    new_params["dsn"], password=new_params.pop("password")
+                )
+
+        conn_params.update({k: unicode2utf8(v) for k, v in new_params.items() if v})
+
+        conn = psycopg2.connect(**conn_params)
+        cursor = conn.cursor()
+        conn.set_client_encoding("utf8")
+
+        self._conn_params = conn_params
+        if self.conn:
             self.conn.close()
         self.conn = conn
         self.conn.autocommit = True
 
-        if dsn:
-            # When we connect using a DSN, we don't really know what db,
-            # user, etc. we connected to. Let's read it.
-            # Note: moved this after setting autocommit because of #664.
-            dsn_parameters = conn.get_dsn_parameters()
-            db = dsn_parameters['dbname']
-            user = dsn_parameters['user']
-            host = dsn_parameters['host']
-            port = dsn_parameters['port']
+        # When we connect using a DSN, we don't really know what db,
+        # user, etc. we connected to. Let's read it.
+        # Note: moved this after setting autocommit because of #664.
+        libpq_version = psycopg2.__libpq_version__
+        dsn_parameters = {}
+        if libpq_version >= 93000:
+            # use actual connection info from psycopg2.extensions.Connection.info
+            # as libpq_version > 9.3 is available and required dependency
+            dsn_parameters = conn.info.dsn_parameters
+        else:
+            try:
+                dsn_parameters = conn.get_dsn_parameters()
+            except Exception as x:
+                # https://github.com/dbcli/pgcli/issues/1110
+                # PQconninfo not available in libpq < 9.3
+                _logger.info("Exception in get_dsn_parameters: %r", x)
+
+        if dsn_parameters:
+            self.dbname = dsn_parameters.get("dbname")
+            self.user = dsn_parameters.get("user")
+            self.host = dsn_parameters.get("host")
+            self.port = dsn_parameters.get("port")
+        else:
+            self.dbname = conn_params.get("database")
+            self.user = conn_params.get("user")
+            self.host = conn_params.get("host")
+            self.port = conn_params.get("port")
 
-        self.dbname = db
-        self.user = user
         self.password = password
-        self.host = host
-        self.port = port
+        self.extra_args = kwargs
 
         if not self.host:
             self.host = self.get_socket_directory()
 
-        cursor.execute("SHOW ALL")
-        db_parameters = dict(name_val_desc[:2] for name_val_desc in cursor.fetchall())
-
-        pid = self._select_one(cursor, 'select pg_backend_pid()')[0]
+        pid = self._select_one(cursor, "select pg_backend_pid()")[0]
         self.pid = pid
-        self.superuser = db_parameters.get('is_superuser') == '1'
+        self.superuser = conn.get_parameter_status("is_superuser") in ("on", "1")
+        self.server_version = conn.get_parameter_status("server_version")
 
         register_date_typecasters(conn)
         register_json_typecasters(self.conn, self._json_typecaster)
         register_hstore_typecaster(self.conn)
 
+    @property
+    def short_host(self):
+        if "," in self.host:
+            host, _, _ = self.host.partition(",")
+        else:
+            host = self.host
+        short_host, _, _ = host.partition(".")
+        return short_host
+
     def _select_one(self, cur, sql):
         """
         Helper method to run a select and retrieve a single field value
@@ -259,20 +338,20 @@ class PGExecute(object):
         else:
             return json_data
 
-
     def failed_transaction(self):
         status = self.conn.get_transaction_status()
         return status == ext.TRANSACTION_STATUS_INERROR
 
-
     def valid_transaction(self):
         status = self.conn.get_transaction_status()
-        return (status == ext.TRANSACTION_STATUS_ACTIVE or
-                status == ext.TRANSACTION_STATUS_INTRANS)
+        return (
+            status == ext.TRANSACTION_STATUS_ACTIVE
+            or status == ext.TRANSACTION_STATUS_INTRANS
+        )
 
-
-    def run(self, statement, pgspecial=None, exception_formatter=None,
-            on_error_resume=False):
+    def run(
+        self, statement, pgspecial=None, exception_formatter=None, on_error_resume=False
+    ):
         """Execute the sql in the database and return the results.
 
         :param statement: A string containing one or more sql statements
@@ -286,29 +365,44 @@ class PGExecute(object):
                execute.
 
         :return: Generator yielding tuples containing
-                 (title, rows, headers, status, query, success)
+                 (title, rows, headers, status, query, success, is_special)
         """
 
         # Remove spaces and EOL
         statement = statement.strip()
         if not statement:  # Empty string
-            yield (None, None, None, None, statement, False)
+            yield (None, None, None, None, statement, False, False)
 
         # Split the sql into separate queries and run each one.
         for sql in sqlparse.split(statement):
             # Remove spaces, eol and semi-colons.
-            sql = sql.rstrip(';')
+            sql = sql.rstrip(";")
+            if not sql:
+                continue
 
             try:
                 if pgspecial:
+                    # \G is treated specially since we have to set the expanded output.
+                    if sql.endswith("\\G"):
+                        if not pgspecial.expanded_output:
+                            pgspecial.expanded_output = True
+                            self.reset_expanded = True
+                        sql = sql[:-2].strip()
+
                     # First try to run each query as special
-                    _logger.debug('Trying a pgspecial command. sql: %r', sql)
-                    cur = self.conn.cursor()
+                    _logger.debug("Trying a pgspecial command. sql: %r", sql)
+                    try:
+                        cur = self.conn.cursor()
+                    except psycopg2.InterfaceError:
+                        # edge case when connection is already closed, but we
+                        # don't need cursor for special_cmd.arg_type == NO_QUERY.
+                        # See https://github.com/dbcli/pgcli/issues/1014.
+                        cur = None
                     try:
                         for result in pgspecial.execute(cur, sql):
                             # e.g. execute_from_file already appends these
-                            if len(result) < 6:
-                                yield result + (sql, True)
+                            if len(result) < 7:
+                                yield result + (sql, True, True)
                             else:
                                 yield result
                         continue
@@ -316,46 +410,45 @@ class PGExecute(object):
                         pass
 
                 # Not a special command, so execute as normal sql
-                yield self.execute_normal_sql(sql) + (sql, True)
+                yield self.execute_normal_sql(sql) + (sql, True, False)
             except psycopg2.DatabaseError as e:
                 _logger.error("sql: %r, error: %r", sql, e)
                 _logger.error("traceback: %r", traceback.format_exc())
 
-                if (self._must_raise(e)
-                        or not exception_formatter):
+                if self._must_raise(e) or not exception_formatter:
                     raise
 
-                yield None, None, None, exception_formatter(e), sql, False
+                yield None, None, None, exception_formatter(e), sql, False, False
 
                 if not on_error_resume:
                     break
+            finally:
+                if self.reset_expanded:
+                    pgspecial.expanded_output = False
+                    self.reset_expanded = None
 
     def _must_raise(self, e):
         """Return true if e is an error that should not be caught in ``run``.
 
-        ``OperationalError``s are raised for errors that are not under the
-        control of the programmer. Usually that means unexpected disconnects,
-        which we shouldn't catch; we handle uncaught errors by prompting the
-        user to reconnect. We *do* want to catch OperationalErrors caused by a
-        lock being unavailable, as reconnecting won't solve that problem.
+        An uncaught error will prompt the user to reconnect; as long as we
+        detect that the connection is stil open, we catch the error, as
+        reconnecting won't solve that problem.
 
         :param e: DatabaseError. An exception raised while executing a query.
 
         :return: Bool. True if ``run`` must raise this exception.
 
         """
-        return (isinstance(e, psycopg2.OperationalError) and
-                (not e.pgcode or
-                 psycopg2.errorcodes.lookup(e.pgcode) != 'LOCK_NOT_AVAILABLE'))
+        return self.conn.closed != 0
 
     def execute_normal_sql(self, split_sql):
         """Returns tuple (title, rows, headers, status)"""
-        _logger.debug('Regular sql statement. sql: %r', split_sql)
+        _logger.debug("Regular sql statement. sql: %r", split_sql)
         cur = self.conn.cursor()
         cur.execute(split_sql)
 
         # conn.notices persist between queies, we use pop to clear out the list
-        title = ''
+        title = ""
         while len(self.conn.notices) > 0:
             title = utf8tounicode(self.conn.notices.pop()) + title
 
@@ -365,7 +458,7 @@ class PGExecute(object):
             headers = [x[0] for x in cur.description]
             return title, cur, headers, cur.statusmessage
         else:
-            _logger.debug('No rows in result.')
+            _logger.debug("No rows in result.")
             return title, None, None, cur.statusmessage
 
     def search_path(self):
@@ -373,26 +466,56 @@ class PGExecute(object):
 
         try:
             with self.conn.cursor() as cur:
-                _logger.debug('Search path query. sql: %r', self.search_path_query)
+                _logger.debug("Search path query. sql: %r", self.search_path_query)
                 cur.execute(self.search_path_query)
                 return [x[0] for x in cur.fetchall()]
         except psycopg2.ProgrammingError:
-            fallback = 'SELECT * FROM current_schemas(true)'
+            fallback = "SELECT * FROM current_schemas(true)"
             with self.conn.cursor() as cur:
-                _logger.debug('Search path query. sql: %r', fallback)
+                _logger.debug("Search path query. sql: %r", fallback)
                 cur.execute(fallback)
                 return cur.fetchone()[0]
 
+    def view_definition(self, spec):
+        """Returns the SQL defining views described by `spec`"""
+
+        template = "CREATE OR REPLACE {6} VIEW {0}.{1} AS \n{3}"
+        # 2: relkind, v or m (materialized)
+        # 4: reloptions, null
+        # 5: checkoption: local or cascaded
+        with self.conn.cursor() as cur:
+            sql = self.view_definition_query
+            _logger.debug("View Definition Query. sql: %r\nspec: %r", sql, spec)
+            try:
+                cur.execute(sql, (spec,))
+            except psycopg2.ProgrammingError:
+                raise RuntimeError("View {} does not exist.".format(spec))
+            result = cur.fetchone()
+            view_type = "MATERIALIZED" if result[2] == "m" else ""
+            return template.format(*result + (view_type,))
+
+    def function_definition(self, spec):
+        """Returns the SQL defining functions described by `spec`"""
+
+        with self.conn.cursor() as cur:
+            sql = self.function_definition_query
+            _logger.debug("Function Definition Query. sql: %r\nspec: %r", sql, spec)
+            try:
+                cur.execute(sql, (spec,))
+                result = cur.fetchone()
+                return result[0]
+            except psycopg2.ProgrammingError:
+                raise RuntimeError("Function {} does not exist.".format(spec))
 
     def schemata(self):
         """Returns a list of schema names in the database"""
 
         with self.conn.cursor() as cur:
-            _logger.debug('Schemata Query. sql: %r', self.schemata_query)
+            _logger.debug("Schemata Query. sql: %r", self.schemata_query)
             cur.execute(self.schemata_query)
             return [x[0] for x in cur.fetchall()]
 
-    def _relations(self, kinds=('r', 'v', 'm')):
+    def _relations(self, kinds=("r", "v", "m")):
         """Get table or view name metadata
 
         :param kinds: list of postgres relkind filters:
@@ -404,14 +527,14 @@ class PGExecute(object):
 
         with self.conn.cursor() as cur:
             sql = cur.mogrify(self.tables_query, [kinds])
-            _logger.debug('Tables Query. sql: %r', sql)
+            _logger.debug("Tables Query. sql: %r", sql)
             cur.execute(sql)
             for row in cur:
                 yield row
 
     def tables(self):
         """Yields (schema_name, table_name) tuples"""
-        for row in self._relations(kinds=['r']):
+        for row in self._relations(kinds=["r"]):
             yield row
 
     def views(self):
@@ -419,10 +542,10 @@ class PGExecute(object):
 
             Includes both views and and materialized views
         """
-        for row in self._relations(kinds=['v', 'm']):
+        for row in self._relations(kinds=["v", "m"]):
             yield row
 
-    def _columns(self, kinds=('r', 'v', 'm')):
+    def _columns(self, kinds=("r", "v", "m")):
         """Get column metadata for tables and views
 
         :param kinds: kinds: list of postgres relkind filters:
@@ -433,13 +556,13 @@ class PGExecute(object):
         """
 
         if self.conn.server_version >= 80400:
-            columns_query = '''
+            columns_query = """
                 SELECT  nsp.nspname schema_name,
                         cls.relname table_name,
                         att.attname column_name,
                         att.atttypid::regtype::text type_name,
                         att.atthasdef AS has_default,
-                        def.adsrc as default
+                        pg_catalog.pg_get_expr(def.adbin, def.adrelid, true) as default
                 FROM    pg_catalog.pg_attribute att
                         INNER JOIN pg_catalog.pg_class cls
                             ON att.attrelid = cls.oid
@@ -451,9 +574,9 @@ class PGExecute(object):
                 WHERE   cls.relkind = ANY(%s)
                         AND NOT att.attisdropped
                         AND att.attnum  > 0
-                ORDER BY 1, 2, att.attnum'''
+                ORDER BY 1, 2, att.attnum"""
         else:
-            columns_query = '''
+            columns_query = """
                 SELECT  nsp.nspname schema_name,
                         cls.relname table_name,
                         att.attname column_name,
@@ -470,44 +593,44 @@ class PGExecute(object):
                 WHERE   cls.relkind = ANY(%s)
                         AND NOT att.attisdropped
                         AND att.attnum  > 0
-                ORDER BY 1, 2, att.attnum'''
+                ORDER BY 1, 2, att.attnum"""
 
         with self.conn.cursor() as cur:
             sql = cur.mogrify(columns_query, [kinds])
-            _logger.debug('Columns Query. sql: %r', sql)
+            _logger.debug("Columns Query. sql: %r", sql)
             cur.execute(sql)
             for row in cur:
                 yield row
 
     def table_columns(self):
-        for row in self._columns(kinds=['r']):
+        for row in self._columns(kinds=["r"]):
             yield row
 
     def view_columns(self):
-        for row in self._columns(kinds=['v', 'm']):
+        for row in self._columns(kinds=["v", "m"]):
             yield row
 
     def databases(self):
         with self.conn.cursor() as cur:
-            _logger.debug('Databases Query. sql: %r', self.databases_query)
+            _logger.debug("Databases Query. sql: %r", self.databases_query)
             cur.execute(self.databases_query)
             return [x[0] for x in cur.fetchall()]
 
     def full_databases(self):
         with self.conn.cursor() as cur:
-            _logger.debug('Databases Query. sql: %r',
-                          self.full_databases_query)
+            _logger.debug("Databases Query. sql: %r", self.full_databases_query)
             cur.execute(self.full_databases_query)
             headers = [x[0] for x in cur.description]
             return cur.fetchall(), headers, cur.statusmessage
 
     def get_socket_directory(self):
         with self.conn.cursor() as cur:
-            _logger.debug('Socket directory Query. sql: %r',
-                          self.socket_directory_query)
+            _logger.debug(
+                "Socket directory Query. sql: %r", self.socket_directory_query
+            )
             cur.execute(self.socket_directory_query)
             result = cur.fetchone()
-            return result[0] if result else ''
+            return result[0] if result else ""
 
     def foreignkeys(self):
         """Yields ForeignKey named tuples"""
@@ -516,7 +639,7 @@ class PGExecute(object):
             return
 
         with self.conn.cursor() as cur:
-            query = '''
+            query = """
                 SELECT s_p.nspname AS parentschema,
                        t_p.relname AS parenttable,
                        unnest((
@@ -543,8 +666,8 @@ class PGExecute(object):
                 JOIN pg_catalog.pg_class      t_c ON t_c.oid = fk.conrelid
                 JOIN pg_catalog.pg_namespace  s_c ON s_c.oid = t_c.relnamespace
                 WHERE fk.contype = 'f';
-                '''
-            _logger.debug('Functions Query. sql: %r', query)
+                """
+            _logger.debug("Functions Query. sql: %r", query)
             cur.execute(query)
             for row in cur:
                 yield ForeignKey(*row)
@@ -552,8 +675,28 @@ class PGExecute(object):
     def functions(self):
         """Yields FunctionMetadata named tuples"""
 
-        if self.conn.server_version > 90000:
-            query = '''
+        if self.conn.server_version >= 110000:
+            query = """
+                SELECT n.nspname schema_name,
+                        p.proname func_name,
+                        p.proargnames,
+                        COALESCE(proallargtypes::regtype[], proargtypes::regtype[])::text[],
+                        p.proargmodes,
+                        prorettype::regtype::text return_type,
+                        p.prokind = 'a' is_aggregate,
+                        p.prokind = 'w' is_window,
+                        p.proretset is_set_returning,
+                        d.deptype = 'e' is_extension,
+                        pg_get_expr(proargdefaults, 0) AS arg_defaults
+                FROM pg_catalog.pg_proc p
+                        INNER JOIN pg_catalog.pg_namespace n
+                            ON n.oid = p.pronamespace
+                LEFT JOIN pg_depend d ON d.objid = p.oid and d.deptype = 'e'
+                WHERE p.prorettype::regtype != 'trigger'::regtype
+                ORDER BY 1, 2
+                """
+        elif self.conn.server_version > 90000:
+            query = """
                 SELECT n.nspname schema_name,
                         p.proname func_name,
                         p.proargnames,
@@ -563,15 +706,17 @@ class PGExecute(object):
                         p.proisagg is_aggregate,
                         p.proiswindow is_window,
                         p.proretset is_set_returning,
+                        d.deptype = 'e' is_extension,
                         pg_get_expr(proargdefaults, 0) AS arg_defaults
                 FROM pg_catalog.pg_proc p
                         INNER JOIN pg_catalog.pg_namespace n
                             ON n.oid = p.pronamespace
+                LEFT JOIN pg_depend d ON d.objid = p.oid and d.deptype = 'e'
                 WHERE p.prorettype::regtype != 'trigger'::regtype
                 ORDER BY 1, 2
-                '''
+                """
         elif self.conn.server_version >= 80400:
-            query = '''
+            query = """
                 SELECT n.nspname schema_name,
                         p.proname func_name,
                         p.proargnames,
@@ -581,15 +726,17 @@ class PGExecute(object):
                         p.proisagg is_aggregate,
                         false is_window,
                         p.proretset is_set_returning,
+                        d.deptype = 'e' is_extension,
                         NULL AS arg_defaults
                 FROM pg_catalog.pg_proc p
-                INNER JOIN pg_catalog.pg_namespace n
-                ON n.oid = p.pronamespace
+                        INNER JOIN pg_catalog.pg_namespace n
+                            ON n.oid = p.pronamespace
+                LEFT JOIN pg_depend d ON d.objid = p.oid and d.deptype = 'e'
                 WHERE p.prorettype::regtype != 'trigger'::regtype
                 ORDER BY 1, 2
-                '''
+                """
         else:
-            query = '''
+            query = """
                 SELECT n.nspname schema_name,
                         p.proname func_name,
                         p.proargnames,
@@ -599,26 +746,28 @@ class PGExecute(object):
                         p.proisagg is_aggregate,
                         false is_window,
                         p.proretset is_set_returning,
+                        d.deptype = 'e' is_extension,
                         NULL AS arg_defaults
                 FROM pg_catalog.pg_proc p
-                INNER JOIN pg_catalog.pg_namespace n
-                ON n.oid = p.pronamespace
+                        INNER JOIN pg_catalog.pg_namespace n
+                            ON n.oid = p.pronamespace
+                LEFT JOIN pg_depend d ON d.objid = p.oid and d.deptype = 'e'
                 WHERE p.prorettype::regtype != 'trigger'::regtype
                 ORDER BY 1, 2
-                '''
+                """
 
         with self.conn.cursor() as cur:
-            _logger.debug('Functions Query. sql: %r', query)
+            _logger.debug("Functions Query. sql: %r", query)
             cur.execute(query)
             for row in cur:
-                  yield FunctionMetadata(*row)
+                yield FunctionMetadata(*row)
 
     def datatypes(self):
         """Yields tuples of (schema_name, type_name)"""
 
         with self.conn.cursor() as cur:
             if self.conn.server_version > 90000:
-                query = '''
+                query = """
                     SELECT n.nspname schema_name,
                            t.typname type_name
                     FROM   pg_catalog.pg_type t
@@ -639,9 +788,9 @@ class PGExecute(object):
                           AND n.nspname <> 'pg_catalog'
                           AND n.nspname <> 'information_schema'
                     ORDER BY 1, 2;
-                    '''
+                    """
             else:
-                query = '''
+                query = """
                     SELECT n.nspname schema_name,
                       pg_catalog.format_type(t.oid, NULL) type_name
                     FROM pg_catalog.pg_type t
@@ -652,17 +801,16 @@ class PGExecute(object):
                           AND n.nspname <> 'information_schema'
                       AND pg_catalog.pg_type_is_visible(t.oid)
                     ORDER BY 1, 2;
-                '''
-            _logger.debug('Datatypes Query. sql: %r', query)
+                """
+            _logger.debug("Datatypes Query. sql: %r", query)
             cur.execute(query)
             for row in cur:
                 yield row
 
-
     def casing(self):
         """Yields the most common casing for names used in db functions"""
         with self.conn.cursor() as cur:
-            query = r'''
+            query = r"""
           WITH Words AS (
                 SELECT regexp_split_to_table(prosrc, '\W+') AS Word, COUNT(1)
                 FROM pg_catalog.pg_proc P
@@ -702,8 +850,8 @@ class PGExecute(object):
             FROM OrderWords
             WHERE LOWER(Word) IN (SELECT Name FROM Names)
             AND Row_Number = 1;
-            '''
-            _logger.debug('Casing Query. sql: %r', query)
+            """
+            _logger.debug("Casing Query. sql: %r", query)
             cur.execute(query)
             for row in cur:
                 yield row[0]
