@@ -1,14 +1,23 @@
+from __future__ import print_function
+
 import os
 import pytest
-from utils import (POSTGRES_HOST, POSTGRES_USER, POSTGRES_PASSWORD, create_db, db_connection,
-                   drop_tables)
+from utils import (
+    POSTGRES_HOST,
+    POSTGRES_PORT,
+    POSTGRES_USER,
+    POSTGRES_PASSWORD,
+    create_db,
+    db_connection,
+    drop_tables,
+)
 import pgcli.pgexecute
 
 
 @pytest.yield_fixture(scope="function")
 def connection():
-    create_db('_test_db')
-    connection = db_connection('_test_db')
+    create_db("_test_db")
+    connection = db_connection("_test_db")
     yield connection
 
     drop_tables(connection)
@@ -23,8 +32,14 @@ def cursor(connection):
 
 @pytest.fixture
 def executor(connection):
-    return pgcli.pgexecute.PGExecute(database='_test_db', user=POSTGRES_USER,
-                                     host=POSTGRES_HOST, password=POSTGRES_PASSWORD, port=None, dsn=None)
+    return pgcli.pgexecute.PGExecute(
+        database="_test_db",
+        user=POSTGRES_USER,
+        host=POSTGRES_HOST,
+        password=POSTGRES_PASSWORD,
+        port=POSTGRES_PORT,
+        dsn=None,
+    )
 
 
 @pytest.fixture
@@ -36,4 +51,4 @@ def exception_formatter():
 def temp_config(tmpdir_factory):
     # this function runs on start of test session.
     # use temporary directory for config home so user config will not be used
-    os.environ['XDG_CONFIG_HOME'] = str(tmpdir_factory.mktemp('data'))
+    os.environ["XDG_CONFIG_HOME"] = str(tmpdir_factory.mktemp("data"))
